{******************************************************************************}
{*    "DirectX.    Delphi".                  *}
{*  BHV. www.bhv.ru                                               *}
{*  .. softgl@chat.ru                                                *}
{******************************************************************************}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AppEvnts,
  DirectXGraphics;

type
  TfrmD3D = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure ApplicationEvents1Minimize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ApplicationEvents1Restore(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FD3D            : IDIRECT3D8;
    FD3DDevice      : IDIRECT3DDEVICE8;
    FD3DVB          : IDIRECT3DVERTEXBUFFER8;
    FActive         : BOOL;
    ThisTickCount   : DWORD;
    LastTickCount   : DWORD;
    function Render : HRESULT;
    function InitD3D : HRESULT;
    function InitPoints : HRESULT;
    function MovePoints : HRESULT;
    procedure Cleanup;
    procedure ErrorOut (const Caption : PChar; const hError : HRESULT);
  public
    { Public declarations }
  end;

type
  TCUSTOMVERTEX = packed record
    X, Y, Z, RHW : Single;
    Color : DWORD;
  end;

const
  MAXPOINTS = 20898;

const
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZRHW or D3DFVF_DIFFUSE;

var
  frmD3D: TfrmD3D;
  Frames : Integer = 0;

  VPoints : Array [0..MAXPOINTS - 1] of TCUSTOMVERTEX;

implementation

{$R *.DFM}

procedure TfrmD3D.Cleanup;
begin
  if Assigned (FD3DVB) then begin
     FD3DVB._Release;
     FD3DVB := nil;
  end;   

  if Assigned (FD3DDevice) then begin
     FD3DDevice._Release;
     FD3DDevice := nil;
  end;

  if Assigned (FD3D) then begin
     FD3D._Release;
     FD3D := nil;
  end;
end;

procedure TfrmD3D.ErrorOut (const Caption : PChar; const hError : HRESULT);
begin
  FActive := False;
  Cleanup;
  MessageBox (Handle, PChar(DXGErrorString (hError)), Caption, 0);
end;

function TfrmD3D.InitD3D : HRESULT;
var
  d3ddm : TD3DDISPLAYMODE;
  d3dpp : TD3DPRESENT_PARAMETERS;
  hRet : HRESULT;
begin
  FD3DDevice := nil;
  FD3DVB := nil;
  FD3D := nil;

  FD3D := Direct3DCreate8(D3D_SDK_VERSION);
  if FD3D = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3D.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  ZeroMemory(@d3dpp, SizeOf(d3dpp));
  with d3dpp do begin
       Windowed := True;
       SwapEffect := D3DSWAPEFFECT_DISCARD;
       BackBufferFormat := d3ddm.Format;
  end;

  Result := FD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, Handle,
                              D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                              d3dpp, FD3DDevice);
end;

var
  PointSize : Single;

function TfrmD3D.Render : HRESULT;
var
  hRet : HRESULT;
begin
  if FD3DDevice = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3DDevice.Clear(0, nil, D3DCLEAR_TARGET, D3DCOLOR_XRGB(255, 255, 255), 0.0, 0);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.BeginScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  FD3DDevice.SetRenderState(D3DRS_POINTSIZE, PDWORD(@PointSize)^);

  hRet := FD3DDevice.DrawPrimitive(D3DPT_POINTLIST, 0, MAXPOINTS);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.EndScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  Result := FD3DDevice.Present(nil, nil, 0, nil);
end;

procedure TfrmD3D.FormCreate(Sender: TObject);
var
  hRet : HRESULT;
begin
  Randomize;
  hRet := InitD3D;
  if Failed (hRet) then ErrorOut ('InitD3D', hRet);
  hRet := InitPoints;
  if Failed (hRet) then ErrorOut ('InitPoint', hRet);

  PointSize := 5.0;
end;

procedure TfrmD3D.ApplicationEvents1Minimize(Sender: TObject);
begin
  FActive := False;
end;

procedure TfrmD3D.FormActivate(Sender: TObject);
begin
  LastTickCount := GetTickCount;
  FActive := True;
end;

var
  Count : Integer = 0;

procedure TfrmD3D.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
  hRet : HRESULT;
begin
  if FActive then begin
     Inc (Frames);
     hRet := Render;
     if FAILED(hRet) then begin
        FActive := False;
        ErrorOut ('Render', hRet);
        Exit;
     end;

     ThisTickCount := GetTickCount;
     if ThisTickCount - LastTickCount > 50 then begin
        Caption := 'FPS = ' + Format('%6.2f',
                                 [frames * 1000 / (ThisTickCount - LastTickCount)]);
        Frames := 0;

        Inc (Count);
        if Count <= 100 then MovePoints;

        LastTickCount := GetTickCount;
     end;
  end;

  Done := False;
end;

procedure TfrmD3D.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then Close else
  if Key = VK_SPACE then begin
     InitPoints;
     PointSize := 5.0;
     Count := 0;
  end;
end;

procedure TfrmD3D.ApplicationEvents1Restore(Sender: TObject);
begin
  FActive := True;
end;

procedure TfrmD3D.FormDestroy(Sender: TObject);
begin
  Cleanup
end;

type
  TStep = packed record
      StepX, StepY : Single;
  end;

var
  Steps : Array [0..MAXPOINTS - 1] of TStep;

function TfrmD3D.InitPoints : HRESULT;
var
  pVertices : PByte;
  hRet : HRESULT;
  i : Integer;
  t : TextFile;
  wrkX, wrkY : Integer;
begin
  AssignFile (t, 'points.txt');
  Reset (t);

  for i := 0 to MAXPOINTS - 1 do begin
      ReadLn (t, wrkX, wrkY);
      with VPoints [i] do begin
         X := random (240);
         Y := random (289);
         Steps [i].StepX := (wrkX - X) / 100;
         Steps [i].StepY := (wrkY - Y) / 100;
         Z := 0.0;
         RHW := 0.0;
         Color := 0;
      end;
  end;

  CloseFile (t);

  hRet := FD3DDevice.CreateVertexBuffer(SizeOf(VPoints),
                                        D3DUSAGE_WRITEONLY, D3DFVF_CUSTOMVERTEX,
                                        D3DPOOL_DEFAULT, FD3DVB);
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DVB.Lock(0, SizeOf(VPoints), pVertices, 0);
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  Move (VPoints, pVertices^, SizeOf(VPoints));

  hRet := FD3DVB.Unlock;
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.SetStreamSource(0, FD3DVB, SizeOf(TCUSTOMVERTEX));
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  Result := FD3DDevice.SetVertexShader(D3DFVF_CUSTOMVERTEX);
end;

function TfrmD3D.MovePoints : HRESULT;
var
  pVertices : PByte;
  hRet : HRESULT;
  i : Integer;
begin
  PointSize := PointSize - 0.04;

  for i := 0 to MAXPOINTS - 1 do begin
      with VPoints [i] do begin
         X := X + Steps [i].StepX;
         Y := Y + Steps [i].StepY;
      end;
  end;

  hRet := FD3DVB.Lock(0, SizeOf(VPoints), pVertices, 0);
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  Move (VPoints, pVertices^, SizeOf(VPoints));

  Result := FD3DVB.Unlock;
end;

end.
